import logging

import Tkinter as tk

from PIL import ImageTk as PILImageTk


DOWNLOAD_DIRECTORY = "./images"

log = logging.getLogger(__name__)


class App(tk.Frame):
    """The TK main application class."""

    TITLE = "9GAG Scraper"
    SEARCH_TERM_LABEL_TEXT = "Search term:"
    SCRAPE_BUTTON_TEXT = "Scrape!"

    def __init__(self, scrape_cb, save_image_cb, master=None):
        tk.Frame.__init__(self, master)
        # Callback functions
        self._scrape_cb = scrape_cb
        self._save_image_cb = save_image_cb
        # TK components
        self._search_term = None
        self._search_term_label = None
        self._search_term_entry = None
        self._scrape_button = None
        self._image_container = None

        self._setup_application()
        self._init_widgets()
        self._position_widgets()

    def _setup_application(self):
        """Setup the application's configuration."""

        self.master.title(App.TITLE)

    def _init_widgets(self):
        """Initialize widgets."""

        self._search_term = tk.StringVar()
        self._search_term_entry = tk.Entry(
                self, textvariable=self._search_term)
        self._search_term_label = tk.Label(
                self, text=App.SEARCH_TERM_LABEL_TEXT)
        self._scrape_button = tk.Button(
                self, text=App.SCRAPE_BUTTON_TEXT, command=self._scrape)
        self._image_container = ImageContainer(
                self._save_image_cb, master=self)

    def _position_widgets(self):
        """Position and pad application widgets."""

        self.grid(column=0, row=0, sticky=(tk.N, tk.W, tk.E, tk.S))
        # First row
        self._search_term_label.grid(column=0, row=0, padx=5, pady=5)
        self._search_term_entry.grid(column=1, row=0, padx=5, pady=5)
        # Second row
        self._scrape_button.grid(column=0, row=1, columnspan=2, padx=5, pady=5)
        # Third row and onwards
        self._image_container.grid(
                column=0, row=2, columnspan=2, sticky=(tk.W, tk.E))

    def _scrape(self):
        """Call scrape callback function."""

        if self._scrape_cb:
            self._scrape_cb()

    def config_bind(self, bind, cb):
        """Configure a keybind to a callback function."""

        self.master.bind(bind, lambda ev: cb(ev))

    def get_search_term(self):
        """Retrieve the search term written by the user."""

        return self._search_term.get()

    def display_images(self, images):
        """Display the images on the GUI."""

        self._image_container.update_images(images)

    def start(self):
        """Start up the TK application."""

        self.mainloop()


class ImageContainer(tk.Frame):
    """A container that displays the images that have been scraped."""

    DEFAULT_RELIEF = "ridge"
    DEFAULT_BORDERWIDTH = 1
    PLACEHOLDER_TEXT = "It's so empty in here..."

    def __init__(self, save_image_cb, master=None, images=None):
        tk.Frame.__init__(
                self, master, borderwidth=ImageContainer.DEFAULT_BORDERWIDTH,
                relief=ImageContainer.DEFAULT_RELIEF)
        self._save_image_cb = save_image_cb
        self._images = images
        self._render()

    def _render(self):
        """Render the images (or the lack thereof)."""

        if not self._images:
            label = tk.Label(self, text=ImageContainer.PLACEHOLDER_TEXT)
            label.grid(column=0, row=0)
            return
        # Render each image in the container
        for idx, img in enumerate(self._images):
            # TODO: Make it span multple rows
            image_container = Image(self, img, self._save_image_cb)
            image_container.grid(column=idx, row=0)
        for child in self.winfo_children():
            child.grid_configure(padx=5, pady=5)

    def _clear_content(self):
        """Clear the contents of the ImageContainer."""

        for child in self.winfo_children():
            child.destroy()

    def update_images(self, images=None):
        """Update the images displayed in the container."""

        self._images = images
        self._clear_content()
        self._render()


class Image(tk.Frame):
    """
    A component that displays an image and a button to save the image to disk.
    """

    ACTIVE_BUTTON_TEXT = "Save"
    DISABLED_BUTTON_TEXT = "Saved"

    def __init__(self, master, image, save_cb):
        tk.Frame.__init__(self, master)
        self._save_cb = save_cb
        self.image = image
        self._tk_img = PILImageTk.PhotoImage(self.image.thumbnail)
        self._tk_img_label = tk.Label(self, image=self._tk_img)
        self._tk_img_label.image = self._tk_img
        self._tk_img_label.grid(column=0, row=0)
        self._save_button = tk.Button(
                self, text=Image.ACTIVE_BUTTON_TEXT, command=self._save)
        self._save_button.grid(column=0, row=1)

    def _save(self):
        """Call the save callback function."""

        if self._save_cb:
            self._save_cb(self)

    def disable_save_button(self):
        """Disable the save button."""

        self._save_button.config(
                state="disabled", text=Image.DISABLED_BUTTON_TEXT)



import unittest

from ..src.nine_gag_scraper import scraper


class WebElementMock(object):

    def __init__(self, enabled=False, displayed=False):
        self.enabled = enabled
        self.displayed = displayed

    def is_displayed(self):
        return self.displayed

    def is_enabled(self):
        return self.enabled

    def click(self):
        pass

    def send_keys(self, *args, **kwargs):
        pass


class WebdriverMock(object):

    def __init__(self):
        self.page_source = None

    def get(self, url):
        self.page_source = """
        <html>
            <body>
                <img
                    src="https://img-9gag-fun.9cache.com/photo/test_img_1.jpg"
                />
                <img
                    src="https://img-9gag-fun.9cache.com/photo/test_img_2.jpg"
                />
                <img
                    src="https://img-9gag-fun.9cache.com/photo/test_img_3.jpg"
                />
            </body>
        </html>
        """

    def find_element(*args, **kwargs):
        css_selector = args[2]
        if css_selector == "button#onetrust-accept-btn-handler":
            return WebElementMock(enabled=True, displayed=True)
        elif css_selector == "div#onetrust-banner-sdk":
            return WebElementMock(enabled=True, displayed=False)

        raise Exception("Searching for unknown element")

    def find_element_by_tag_name(*args, **kwargs):
        return WebElementMock()

    def quit(self):
        pass


class TestNineGagScraper(unittest.TestCase):
    """Test cases for the NineGagScraper class."""

    def setUp(self):
        self.driver = WebdriverMock()
        self.scraper = scraper.NineGagScraper(self.driver)

    def tearDown(self):
        self.scraper.close()
        self.scraper = None
        self.driver = None

    def test_find_images(self):
        images = list(self.scraper._find_images())
        self.assertEqual(len(images), 3)


if __name__ == '__main__':
    unittest.main()



import io
import time
import logging
import urlparse

import requests
import bs4

from PIL import Image as PILImage
from selenium import webdriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys


log = logging.getLogger(__name__)


class ScrapedImage(object):
    """
    ScrapedImage is an object that represents a scraped image.

    It is useful to contain both the PIL.Image.Image object and its thumbnail
    without the need to copy the image in different places.
    """

    DEFAULT_THUMBNAIL_SIZE = (128, 128)

    def __init__(self, data, name, thumbnail_size=None):
        # Generate full-sized image
        self.image = PILImage.open(io.BytesIO(data))
        self.name = name
        # Generate its thumbnail, but DO NOT overwrite the original image
        if not thumbnail_size:
            thumbnail_size = ScrapedImage.DEFAULT_THUMBNAIL_SIZE
        self.thumbnail = self.image.copy()
        self.thumbnail.thumbnail(thumbnail_size)

    def save(self, *args, **kwargs):
        """Save the image on disk."""

        return self.image.save(*args, **kwargs)


class NineGagScraper(object):
    """A very badly written 9gag image scraper."""

    BASE_ADDRESS = "https://9gag.com"
    SEARCH_ADDRESS_FORMAT = BASE_ADDRESS + "/search?query={}"

    ONETRUST_ACCEPT_BTN_SELECTOR = "button#onetrust-accept-btn-handler"
    ONETRUST_BANNER_SELECTOR = "div#onetrust-banner-sdk"
    ONETRUST_WAIT_TIMEOUT = 20

    DEFAULT_SEARCH_TERM = "kittens"  # Cause they're cute!
    DEFAULT_SCROLL_COUNT = 2

    IMAGE_URL_PREFIX = "https://img-9gag-fun.9cache.com/photo/"

    def __init__(self, driver):
        self._driver = driver
        self._driver.get(NineGagScraper.BASE_ADDRESS)
        self._accept_onetrust_cookies()

    def scrape_images(self, search_term=None):
        """
        Perform the scraping of images and return the results which were found.
        """

        if not search_term:
            search_term = NineGagScraper.DEFAULT_SEARCH_TERM
        search_url = NineGagScraper.SEARCH_ADDRESS_FORMAT.format(search_term)
        self._driver.get(search_url)
        self._scroll_to_bottom(count=NineGagScraper.DEFAULT_SCROLL_COUNT)
        for img_url in self._find_images():
            print("Found img_url: {}".format(img_url))
            # Get the name of the file as it is stored in 9GAG
            parsed_url = urlparse.urlparse(img_url)
            img_name = parsed_url.path.rsplit("/", 1)[-1]
            log.info("Downloading image '%s'", img_name)
            res = requests.get(img_url)
            if not res.ok:
                log.warning("Couldn't download '%s'. Response: (%s %s) %s",
                            img_url, res.status_code, res.reason, res.text)
                continue
            image = ScrapedImage(res.content, img_name)
            log.info("Successfully downloaded '%s'", img_url)
            yield image

    def _accept_onetrust_cookies(self, timeout=20):
        """
        Accept onetrust's cookies and wait for the banner to be invisible.
        """

        onetrust_btn = (
                By.CSS_SELECTOR,
                NineGagScraper.ONETRUST_ACCEPT_BTN_SELECTOR)
        onetrust_banner = (
                By.CSS_SELECTOR,
                NineGagScraper.ONETRUST_BANNER_SELECTOR)
        # Wait for the accept button to be clickable
        btn_appear_wait = WebDriverWait(self._driver, timeout)
        btn_appear_wait.until(
            expected_conditions.element_to_be_clickable(onetrust_btn)).click()
        # Now wait for the banner to disappear
        banner_disappear_wait = WebDriverWait(self._driver, timeout)
        banner_disappear_wait.until(
            expected_conditions.invisibility_of_element(onetrust_banner))

    def _scroll_to_bottom(self, count=1):
        """Scroll to the bottom of the web page count times."""

        html_elem = self._driver.find_element_by_tag_name("html")
        for _ in range(count):
            log.debug("Scrolling selenium to bottom of page")
            html_elem.send_keys(Keys.END)
            # TODO - try and find a better way to wait for the scroll to finish
            time.sleep(2)

    def _find_images(self):
        """Find the images in the web page that are a part of a post."""

        soup = bs4.BeautifulSoup(self._driver.page_source, "html.parser")
        for img in soup.findAll("img"):
            if NineGagScraper.IMAGE_URL_PREFIX in img["src"]:
                yield img["src"]

    def close(self):
        """Close and cleanup the scraper."""

        self._driver.quit()
